import xml.etree.ElementTree as ET
import pandas as pd


class Feature_Extractor():
    all_features = [
        'file_name','ACCESS_NETWORK_STATE','WRITE_EXTERNAL_STORAGE','READ_PHONE_STATE','WAKE_LOCK','ACCESS_FINE_LOCATION','RECEIVE_BOOT_COMPLETED',
        'VIBRATE','SEND_SMS','RECEIVE_SMS','RESTART_PACKAGES','READ_SMS','INTERNET','CCESS_FINE_LOCATION','READ_CONTACTS','READ_LOGS','GET_TASKS','WRITE_SETTINGS',
        'CHANGE_WIFI_STATE','BLUETOOTH_ADMIN','BLUETOOTH','DEFAULT','VIEW','BOOT_COMPLETED','HOME','screen_portrait','location','location_network','location_gps',
        'wifi','telephony','bluetooth','Cipher','getPackageInfo','printStackTrace','Read_Write_External_Storage','HttpPost','getDeviceId','Obfuscation_Base64',
        'Execution_of_external_commands','getSubscriberId','getSimCountryIso','sendSMS','getActiveNetworkInfo','WebView','Context_startActivity',
        'ContentResolver_query','Notify','HttpURLConnection','DefaultHttpClient','sendTextMessage','Sms Receiver','Boot Receiver','Telephony.SMS_RECEIVED',
        'admob.com','schemas.android.com','appspot.com,googleapis.com','androiddoctor.com','sgadtracker.com','rsigma.com','paypal.com'
    ]
    xml_features = ['intent', 'service', 'provider', 'receiver', 'permission']
    txt_features = ['intent', 'service', 'provider', 'receiver', 'permission', 'call', 'api_call', 'feature']
    out_csv = None
    

    def __init__(self, out_path=None):
        self.out_csv = pd.DataFrame(columns=self.all_features)


    def find_selected_xml_nodes(self, node: ET.Element, selected_nodes):
        for feature in self.xml_features:
            if feature in node.tag.lower():
                selected_nodes.append(node)
                break

        for child in node:
            self.find_selected_xml_nodes(child, selected_nodes)


    def extract_file_features(self, selected_nodes) -> set:
        file_features = set()

        for node in selected_nodes:  
            if([elem for elem, _ in node.attrib.items() if elem.endswith('name')]):
                file_features.add(node.attrib['{http://schemas.android.com/apk/res/android}name'].split('.')[-1].lower())
            else:
                for child in node:
                    if([elem for elem, _ in child.attrib.items() if elem.endswith('name')]):
                        file_features.add(child.attrib['{http://schemas.android.com/apk/res/android}name'].split('.')[-1].lower())
        return file_features


    def write_in_features_table(self, name, file_features):
        table_row = [name.split('/')[-1]]
        for feature in self.out_csv.columns[1:]:
            if feature.lower() in file_features:
                table_row.append(1)
            else:
                table_row.append(0)

        self.out_csv.loc[len(self.out_csv)] = table_row


    def save_csv_file(self, file_name='out.csv'):
        self.out_csv.to_csv(file_name, index=False)


    def extract_xml_features(self, file):
        selected_nodes: ET.Element = []

        tree = ET.parse(file)
        root = tree.getroot()

        self.find_selected_xml_nodes(root, selected_nodes)
        file_features = self.extract_file_features(selected_nodes)
        self.write_in_features_table(file, file_features)


    def extract_all_xml_features(self, file):
        selected_nodes: ET.Element = []

        tree = ET.parse(file)
        root = tree.getroot()

        self.find_selected_xml_nodes(root, selected_nodes)
        file_features = self.extract_file_features(selected_nodes)
        return list(file_features)


    def extract_raw_features(self, file):
        file_features = []
        with open(file) as f:
            for line in f:
                feature_type = line.strip().split('::')[0]
                feature_exp = line.strip().split('::')[-1]
                if feature_type in self.txt_features:
                    if '->' in feature_exp:
                        file_features.append(feature_exp.split('->')[-1].lower())
                    else:
                        file_features.append(feature_exp.split('.')[-1].lower())
                elif feature_type == 'url':
                    file_features.append(feature_exp.lower())

        self.write_in_features_table(file, file_features)


    def extract_all_raw_features(self, file):
        file_features = []
        with open(file) as f:
            for line in f:
                feature_type = line.strip().split('::')[0]
                feature_exp = line.strip().split('::')[-1]
                if feature_type in self.txt_features:
                    if '->' in feature_exp:
                        file_features.append(feature_exp.split('->')[-1].lower())
                    else:
                        file_features.append(feature_exp.split('.')[-1].lower())
                elif feature_type == 'url':
                    file_features.append(feature_exp.lower())

        return file_features

    @staticmethod
    def save_all_features_to_csv(all_features: list, file_name='out_all.csv'):
        headers = set()
        headers.add('file_name')
        for fList in all_features:
            for feature in fList[:-1]:
                headers.add(feature.lower())

        headers = list(headers)
        headers.sort()

        out_csv = pd.DataFrame(columns=headers)
        
        for fList in all_features:
            table_row = [fList[-1]]
            
            for feature in headers[1:]:
                if feature.lower() in fList:
                    table_row.append(1)
                else:
                    table_row.append(0)
    
            out_csv.loc[len(out_csv)] = table_row

        out_csv.to_csv(file_name, index=False)
